<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="text"/>

  <xsl:include href="regexp.xsl"/>

<!-- General definitions -->

  <xsl:strip-space elements="*"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>

<!-- Toplevel -->

  <xsl:template match="/">
    <xsl:apply-templates select="lex"/>
  </xsl:template>

  <xsl:template match="lex">
    <xsl:apply-templates select="declarations"/>
    <xsl:value-of select="$newline"/>
    <xsl:text>%%</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:apply-templates select="rules"/>
    <xsl:value-of select="$newline"/>
    <xsl:text>%%</xsl:text>
    <xsl:apply-templates select="definitions"/>
    <xsl:value-of select="$newline"/>
  </xsl:template>


<!-- Declarations -->

  <xsl:template match="declarations">
    <xsl:apply-templates select="option|inclusive_states|exclusive_states|code_declaration|abbreviation"/>
  </xsl:template>

  <xsl:template match="inclusive_states">
    <xsl:text>%s</xsl:text>
    <xsl:for-each select="state_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="exclusive_states">
    <xsl:text>%s</xsl:text>
    <xsl:for-each select="state_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/> 
  </xsl:template>

  <xsl:template match="code_declaration">
    <xsl:text>%{</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="."/>
    <xsl:value-of select="$newline"/>
    <xsl:text>%}</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="option">
    <xsl:text>%option </xsl:text>
    <xsl:value-of select="."/>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="abbreviation">
    <xsl:value-of select="id"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="regexp" />
    <xsl:value-of select="$newline"/>
  </xsl:template>

<!-- Rules (inherit rules for regexps) -->

  <xsl:template match="rules">
    <xsl:apply-templates select="rule|scope"/>
  </xsl:template>

  <xsl:template match="rule">
    <xsl:apply-templates select="scon"/>
    <xsl:apply-templates select="regexp"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="action"/>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="scon">
    <xsl:text>&lt;</xsl:text>
    <xsl:for-each select="state_id">
      <xsl:value-of select="."/>
      <xsl:if test="position() != last()">
        <xsl:text>,</xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:text>&gt;</xsl:text>
  </xsl:template>

  <xsl:template match="any_state">
    <xsl:text>*</xsl:text>
  </xsl:template>

  <xsl:template match="scope">
    <xsl:apply-templates select="scon"/>
    <xsl:text>{</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:apply-templates select="rule|scope"/>
    <xsl:text>}</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="action">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="fallthru">
    <xsl:text>|</xsl:text>
  </xsl:template>

<!-- Definitions -->

  <xsl:template match="ccode">
    <xsl:text>{</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>}</xsl:text>
  </xsl:template>

    

</xsl:stylesheet>

