<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="text"/>

  <xsl:template match="dot">
    <xsl:text>.</xsl:text>
  </xsl:template>

  <xsl:template match="class">
    <xsl:text>[</xsl:text>
    <xsl:if test="@negated = 'true'">
      <xsl:text>^</xsl:text>
    </xsl:if>
    <xsl:apply-templates select="char|range|esc|expr"/>
    <xsl:text>]</xsl:text>
  </xsl:template>

  <xsl:template match="expr">
    <xsl:text>[:</xsl:text>
    <xsl:value-of select="@val"/>
    <xsl:text>:]</xsl:text>
  </xsl:template>

  <xsl:template match="star">
    <xsl:apply-templates select="./*"/>
    <xsl:text>*</xsl:text>
  </xsl:template>

  <xsl:template match="plus">
    <xsl:apply-templates select="./*"/>
    <xsl:text>+</xsl:text>
  </xsl:template>

  <xsl:template match="opt">
    <xsl:apply-templates select="./*"/>
    <xsl:text>?</xsl:text>
  </xsl:template>

  <xsl:template match="pow">
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>{</xsl:text>
    <xsl:apply-templates select="exact|atleast|between"/>
    <xsl:text>}</xsl:text>
  </xsl:template>

  <xsl:template match="followed_by">
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>/</xsl:text>
    <xsl:apply-templates select="*[2]"/>
  </xsl:template>

  <xsl:template match="ref">
    <xsl:text>{</xsl:text>
    <xsl:apply-templates select="./*"/>
    <xsl:text>}</xsl:text>
  </xsl:template>

  <xsl:template match="char">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="quot">
    <xsl:text>"</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>"</xsl:text>
  </xsl:template>

  <xsl:template match="esc">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="seq">
    <xsl:text>(</xsl:text>
    <xsl:apply-templates select="./*"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="alt">
    <xsl:text>(</xsl:text>
    <xsl:for-each select="./*">
      <xsl:text>(</xsl:text>
      <xsl:apply-templates select="."/>
      <xsl:text>)</xsl:text>
      <xsl:if test="position() != last()">
        <xsl:text>|</xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="bol">
    <xsl:text>^</xsl:text>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="eol">
    <xsl:text>$</xsl:text>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="eof">
    <xsl:text>&lt;&lt;EOF&gt;&gt;</xsl:text>
  </xsl:template>

  <xsl:template match="exact">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="atleast">
    <xsl:value-of select="."/>
    <xsl:text>,</xsl:text>
  </xsl:template>

  <xsl:template match="between">
    <xsl:value-of select="num[1]"/>
    <xsl:text>,</xsl:text>
    <xsl:value-of select="num[2]"/>
  </xsl:template>

  <xsl:template match="num">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="range">
    <xsl:value-of select="*[1]"/>
    <xsl:text>-</xsl:text>
    <xsl:value-of select="*[2]"/>
  </xsl:template>    

  <xsl:template match="id">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="unparsed">
    <xsl:value-of select="."/>
  </xsl:template>

</xsl:stylesheet>