stylesheet [version="1.0"];
output [method="xml",indent="yes"];

strip-space "*";

let newline = {"
"};

"/" -> {
apply-templates to "xlex"
};

"xlex" -> {
	<lex> {
		<declarations> {
			<option> {"stack yymore noyywrap full ecs"},
			<code_declaration> {
"
#define YYLEX_OUT
#include &quot;xmlbase.h&quot;
#include &quot;y.tab.h&quot;
"
			},
			call-template "token_name_definition",
			call-template "counter_definitions",
			call-template "symtbl_definitions",
			apply-templates to "declarations"
		},
		<rules> {
		apply-templates to "rules"
		},
		<definitions> {
		}
	}
};

"declarations" -> {
	apply-templates to "./*"
};

"inclusive_states|exclusive_states|abbreviation" -> {
	copy-of "."
};

"token_declaration|counter_declaration" -> {};


symtbl_definitions -> {
	<code_declaration> {
		for-each "/xlex/declarations/symtbl_declaration/symtbl_id" {
			"struct symtbl ",
			apply-templates to ".",
			"_struct = {0,0,NULL};",
			value-of "$newline",
			"symtbl_t ",
			apply-templates to ".",
			" = &amp;",
			apply-templates to ".",
			"_struct;",
			value-of "$newline"
		}
	}
};


counter_definitions -> {
	<code_declaration> {
		for-each "/xlex/declarations/counter_declaration/counter_id" {
			"int ",
			apply-templates to ".",
			" = 0;",
			value-of "$newline"
		}
	}
};

token_name_definition -> {
	<code_declaration> {
"
const char* token_names[] = {NULL,",
		for-each "/xlex/declarations/token_declaration/token_id" {
			"&quot;",
			value-of ".",
			"&quot;",
			if "position() != last()" {
				", "
			}
		},
	"};
"
	}
};

"rules" -> {
	apply-templates to "rule|scope"
};

"rule" -> {
	copy {
		copy-of "scon",
		copy-of "regexp",
		<action> {
			apply-templates to "action"
		}
	}
};

"scope" -> {
	copy {copy-of "scon",
		apply-templates to "rule|scope"
	}
};

"then|else" -> {
	apply-templates
};

"action" -> {
	apply-templates
};

"fallthru" -> {
	<fallthru> {}
};

"action_code" -> {
	<ccode> {
		apply-templates
		}
};

"assign" -> {
	apply-templates to "counter_id",
	" = ",
	value-of "num",
	";"
};

"begin" -> {
	"BEGIN(",
	value-of "state_id",
	");"
};

"push" -> {
	"PUSH(",
	value-of "state_id",
	");"
};

"pop" -> {
	"POP;"
};

"token" -> {
	"TOKEN(",
	apply-templates to "token_id",
	");"
};

"tokentext" -> {
	"TOKENTEXT(",
	apply-templates to "token_id",
	");"
};

"start" -> {
	"START(",
	apply-templates to "token_id",
	");"
};

"continue" -> {
	"CONTINUE;"
};

"end" -> {
	"END;"
};

"skip" -> {};

"echo" -> {
	"ECHO;"
};

"cut" -> {
	"yytext += ",
	apply-templates to "num",
";"
};

"trim" -> {
	"yytext[yyleng-",
	apply-templates to "num",
	"] = '\0';"
};

"incr" -> {
	apply-templates to "counter_id",
	"++;"
};

"decr" -> {
	apply-templates to "counter_id",
	"--;"
};

"error" -> {
	"ERROR(&quot;",
	value-of ".",
	"&quot;);"
};

"add" -> {
	"ADD(",
	apply-templates to "symtbl_id",
	",yytext);"
};

"remove" -> {
	"REMOVE(",
	apply-templates to "symtbl_id",
	",yytext);"
};

"symtbl_id" -> {
	"SYMTBL_",
	value-of "."
};



"token_id" -> {
	"TOK_",
	value-of "."
};

"counter_id" -> {
	"CNTR_",
	value-of "."
};

"if" -> {
	"if(",
	apply-templates to "test",
	") {",
	apply-templates to "then",
	"} else {",
	apply-templates to "else",
	"}"
};

"exp|test" -> {
	apply-templates
};

"equals|nequals|lt|lte|gt|gte" -> {
	apply-templates to "exp[1]",
	" ",
	apply-templates to "." mode symbol,
	" ",
	apply-templates to "exp[2]"
};

"equals" mode symbol -> {
	"=="
};

"nequals" -> {
	"!="
};

"lt" -> {
	"&lt;"
};

"lte" -> {
	"&lt;="
};

"gt" -> {
	"&gt;"
};

"gte" -> {
	"&gt;="
};


"mem" -> {
	"MEMBER(",
	apply-templates to "symtbl_id",
	",yytext)"
};

