stylesheet [version="1.0"];

output [method="text"];
strip-space "*";
preserve-space "xsl:text";

let newline = {"&#10;"};

"/" -> {apply-templates to "xsl:stylesheet"};

"xsl:stylesheet" -> {
	"stylesheet [",
	for-each "@*" {
		value-of "name()",
		"=&quot;",
		value-of ".",
		"&quot;"
	},
	"];",
	value-of "$newline",
	for-each "*" {
		apply-templates to ".",
		";",
		value-of "$newline",
		value-of "$newline"
	}
};



"xsl:output" -> {
	"output [",
	for-each "@*" {
		value-of "name()",
		"=&quot;",
		value-of ".",
		"&quot;",
		if "position() != last()" {
			","
		}
	},
	"]"
};

"xsl:decimal-format" -> {
	"decimal-format [",
	for-each "@*" {
		value-of "name()",
		"=&quot;",
		value-of ".",
		"&quot;",
		if "position() != last()" {
			","
		}
	},
	"]"
};


"xsl:include" -> {
	"include &quot;",
	value-of "@href",
	"&quot;"
};

"xsl:import" -> {
	"import &quot;",
	value-of "@href",
	"&quot;"
};

"xsl:preserve-space" -> {
	"preserve-space &quot;",
	value-of "@elements",
	"&quot;"
};

"xsl:strip-space" -> {
	"strip-space &quot;",
	value-of "@elements",
	"&quot;"
};

"xsl:variable" -> {
	"let ", 
	value-of "@name",
	" = ",
	choose {
		when "boolean(child::*)" {
			"{",
			apply-templates to "child::*",
			"}"
		};
		when "boolean(@select)" {
			"&quot;",
			value-of "@select",
			"&quot;"
		}
	}
};
	
"xsl:with-param" -> {
	"with-param ",
	value-of "@name",
	" = ",
	choose {
		when "boolean(child::*)" {
			"{",
			apply-templates to "child::*",
			"}"
		};
		when "boolean(@select)" {
			"&quot;",
			value-of "@select",
			"&quot;"
		}
	}
};


"xsl:template" -> {
	if "boolean(@name)" {
		value-of "@name",
		" "
	},
	if "boolean(@match)" {
		"&quot;",
		value-of "@match",
		"&quot; "
	},
	if "boolean(@priority)" {
		"priority ",
		value-of "@match",
		" "
	},
	if "boolean(@mode)" {
		"mode ",
		value-of "@match",
		" "
	},
	"-> {",
	value-of "$newline",
	for-each "*" {
		apply-templates to ".",
		if "position() != last()" {
			","
		},
		value-of "$newline"
	},
	"}"
};

"xsl:apply-templates" -> {
	"apply-templates",
	if "boolean(@select)" {
		" to ", "&quot;",value-of "@select","&quot;"
	}
};

"xsl:call-template" -> {
	"call-template ",
	"&quot;",
	value-of "@name",
	"&quot;",
	if "boolean(child::*)" {
		" { ",
		apply-templates to "*",
		" } "
	}
};

"xsl:choose" -> {
	"choose {",
	value-of "$newline",
	for-each "*" {
		apply-templates to ".",
		if "position() != last()" {
			";"
		},
		value-of "$newline"
	}
};

"xsl:for-each" -> {
	"for-each ",
	"&quot;",
	value-of "@select",
	"&quot; {",
	value-of "$newline",
	for-each "sort" {
		apply-templates to ".",
		if "position() != last()" {
			","
		},
		value-of "$newline"
	},
	for-each "*" {
		if "not(boolean(xsl:sort))" {
			apply-templates to ".",
			if "position() != last()" {
				","
			},
			value-of "$newline"
		}
	},
	"}"
};

"xsl:if" -> {
	"if ",
	"&quot;",
	value-of "@test",
	"&quot; {",
	value-of "$newline",
	for-each "*" {
		apply-templates to ".",
		if "position() != last()" {
			","
		},
		value-of "$newline"
	},
	"}"
};


"xsl:attribute" -> {
	"@",
	if "boolean(@namespace)" {
		value-of "@namespace",
		":"
	},
	value-of "@name",
	" {",
	for-each "*" {
		apply-templates to ".",
		if "position() != last()" {
			",",
			value-of "$newline"
		}
	},
	"}"
};

"xsl:comment" -> {
	"comment",
	" {",
	for-each "*" {
		apply-templates to ".",
		if "position() != last()" {
			","
		},
		value-of "$newline"
	},
	"}"
};

"xsl:copy" -> {
	"copy", 
	" {",
	for-each "*" {
		apply-templates to ".",
		if "position() != last()" {
			","
		},
		value-of "$newline"
	},
	"}"
};

"xsl:copy-of" -> {
	"copy-of ",
	"&quot;",
	value-of "@select",
	"&quot;"
};

"xsl:element" -> {
	"&lt;",
	if "boolean(@namespace)" {
		value-of "@namespace",
		":"
	},
	value-of "@name",
	"&gt; {",
	value-of "$newline",
	for-each "*" {
		apply-templates to ".",
		if "position() != last()" {
			","
		},
		value-of "$newline"
	},
	"}"
};

"xsl:text" -> {
	"&quot;",
	value-of ".",
	"&quot;"
};


"xsl:value-of" -> {
	"value-of ",
	"&quot;",
	value-of "@select",
	"&quot;"
};

"xsl:sort" -> {
	"sort [",
	for-each "@*" {
		value-of "name()",
		"=&quot;",
		value-of ".",
		"&quot;",
		if "position() != last()" {
			","
		}
	},
	"]"
};

"*" -> {
	"&lt;",
	if "boolean(@namespace)" {
		value-of "@namespace",
		":"
	},
	value-of "name()",
	for-each "@*" {
		" ",
		value-of "name()",
		"=&quot;",
		value-of ".",
		"&quot;"
	},
	"&gt; {",
	value-of "$newline",
	for-each "*|./text()" {
		apply-templates to ".",
		if "position() != last()" {
			","
		},
		value-of "$newline"
	},
	"}"
};


"text()" -> {	
	"&quot;",
	value-of ".",
	"&quot;"
};