%option stack yymore noyywrap full ecs
%{

#define YYLEX_OUT 

#include "xmlbase.h"
#include "y.tab.h"

int CNTR_lineno = 0;

const char* token_names[] = {
NULL,
"START_DEC",
"TYPE_DEC",
"TOKEN_DEC",
"SYMTBL_DEC",
"BAR",
"SEMI",
"SECTEND",
"LIST_TY",
"STRING_TY",
"ELEMENT_TY",
"ATTRIBUTE_TY",
"CONTENT_TY",
"LBRACE",
"RBRACE",
"LANGLE",
"RANGLE",
"LPAREN",
"RPAREN",
"NL",
"COLON",
"COMMA",
"PREC",
"EOF_TOK",
"EMPTY_TM",
"CONCAT_TM",
"ELEMENT_TM",
"TEXT_TM",
"ATTRIBUTE_TM",
"NIL_TM",
"APPEND_TM",
"CONS_TM",
"LIST_TM",
"REV_TM",
"ERROR_TM",
"VAR_TM",
"ACCEPT_TM",
"LITERAL_TM",
"ASSOC_DEC",
"NONTERM_ID",
"TOKEN_ID",
"CHAR",
"STRING_TM",
"NUM_TM",
"ID",
"SYMTBL_ID",
"DOT",
"COLONCOLON",
"PLUSPLUS",
"LBRACK",
"RBRACK",
"EQUALS",
"ADD_TM",
"REMOVE_TM",
"ERROR_MATCH",
"BANG",
"AT"
     };

       
%}

%s SECT2 TYPE LITERAL TERM COMMENT SYMTBL EXIT

WS		[[:blank:]]+
OPTWS		[[:blank:]]*
NOT_WS		[^[:blank:]\n]

NL		\r?\n

NAME		([[:alpha:]_][[:alnum:]_-]*)
TNAME           ([[:upper:]][[:alnum:]_-]*)
NTNAME          ([[:lower:]][[:alnum:]_-]*)
NSNAME          (({NAME}[:])?{NAME})

NUM             ([[:digit:]]+)
%%
<INITIAL>{
	"/*"		{PUSH( COMMENT );}
        ^"%token"       {TOKEN(TOKEN_DEC);}
        ^"%left"        {CUT(1);
                         TOKENTEXT(ASSOC_DEC);}
        ^"%right"       {CUT(1);
                         TOKENTEXT(ASSOC_DEC);}
        ^"%nonassoc"    {CUT(1);
                         TOKENTEXT(ASSOC_DEC);}
        ^"%start"       {TOKEN(START_DEC);}
        ^"%type"        {TOKEN(TYPE_DEC);}
        ^"%symtbl"      {PUSH(SYMTBL);TOKEN(SYMTBL_DEC);}
	"<" 		{PUSH(TYPE);TOKEN(LANGLE);}
	{WS}		SKIP; /* discard */
	{NTNAME}	TOKENTEXT(NONTERM_ID);
	{TNAME}		TOKENTEXT(TOKEN_ID);

	^"%%".*		{
			BEGIN(SECT2);
			TOKEN(SECTEND);
			}

	^"%"[^sxtrln].*	        {ERROR("unrecognized '%' directive");}
	.		{ERROR("illegal character");}

	^{OPTWS}{NL}	{INCR(CNTR_lineno);TOKEN(NL);} /* allows blank lines in section 1 */
	{OPTWS}{NL}	{INCR(CNTR_lineno);TOKEN(NL);} /* maybe end of comment line */
}

<TYPE>{
	"/*"		{PUSH( COMMENT );}
	">"		{POP;
			 TOKEN(RANGLE);}
	"string"        {TOKEN(STRING_TY);}
	"element"       {TOKEN(ELEMENT_TY);}
	"attribute"     {TOKEN(ATTRIBUTE_TY);}
	"content"       {TOKEN(CONTENT_TY);}
	"list"          {TOKEN(LIST_TY);}
	{WS}            SKIP; /* discard */
}

<SYMTBL>{
	"/*"		{PUSH( COMMENT );}
	{TNAME}		{TOKENTEXT(SYMTBL_ID);}
	{NTNAME}	{TOKENTEXT(SYMTBL_ID);}
	{WS}		{SKIP;}
	{NL}		{POP;TOKEN(NL);}
}

<LITERAL>{
	[^"\n]		{CONTINUE;}
	\"		{POP;TRIM(1);END;}

	{NL}		{
			ERROR( "missing quote" );
			}
}
<COMMENT>{
	"*/"		{POP;}
	"*"		{}
	[^*\n]+		{}
	[^*\n]*{NL}	{INCR(CNTR_lineno);}
}




<SECT2>{
	"/*"		{PUSH( COMMENT );}
	{NL}		INCR(CNTR_lineno);
	{WS}    	SKIP;
	"|"		TOKEN(BAR);
	"{"		PUSH(TERM);TOKEN(LBRACE);
        "error"         {TOKEN(ERROR_MATCH);}
	{NTNAME}	{TOKENTEXT(NONTERM_ID);}
	{TNAME}		TOKENTEXT(TOKEN_ID);
	";"		TOKEN(SEMI);
	":" 		TOKEN(COLON);
	"%prec"		TOKEN(PREC);
	.               ERROR("illegal character");
        <<EOF>>         BEGIN(EXIT);TOKEN(EOF_TOK);
}

<EXIT>{
        <<EOF>>        ERROR("unexpected EOF");
}




<TERM>{
	"/*"		{PUSH( COMMENT );}
	"}"		{POP; TOKEN(RBRACE);}
	"("             {TOKEN(LPAREN);}
	")"		{TOKEN(RPAREN);}
	"["    		{TOKEN(LBRACK);}
	"]"		{TOKEN(RBRACK);}
	"<"    		{TOKEN(LANGLE);}
	">"		{TOKEN(RANGLE);}
	"="		{TOKEN(EQUALS);}
	"."		{TOKEN(DOT);}
	"++"		{TOKEN(PLUSPLUS);}
	"::"		{TOKEN(COLONCOLON);}
	";"		{TOKEN(SEMI);}
	","		{TOKEN(COMMA);}
	"$"		{TOKEN(VAR_TM);}
        "!"             {TOKEN(BANG);}
        "@"             {TOKEN(AT);}
	["]["]          {TOKEN(EMPTY_TM);}
	["]             {PUSH(LITERAL);START(STRING_TM);}
        "concat"	{TOKEN(CONCAT_TM);}
	"element"	{TOKEN(ELEMENT_TM);}
	"text"	 	{TOKEN(TEXT_TM);}
        "literal"       {TOKEN(LITERAL_TM);}
	"attribute"	{TOKEN(ATTRIBUTE_TM);}
	"nil"           {TOKEN(NIL_TM);}
	"append"        {TOKEN(APPEND_TM);}
	"list"          {TOKEN(LIST_TM);}
	"rev"		{TOKEN(REV_TM);}
	"error"		{TOKEN(ERROR_TM);}
	"accept"	{TOKEN(ACCEPT_TM);}
        "add"           {TOKEN(ADD_TM);}
        "remove"        {TOKEN(REMOVE_TM);}
	{NUM}		{TOKENTEXT(NUM_TM);}
	{NSNAME}        {TOKENTEXT(ID);}
	{NL}		{INCR(CNTR_lineno);}
        {WS}            SKIP;
	. 		{ERROR("illegal character");}
}

<COMMENT,LITERAL,TERM><<EOF>>	{
			ERROR( "EOF encountered inside an action, literal, or comment");
			}



%%
