<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="text"/>

<!-- General definitions -->

  <xsl:strip-space elements="*"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>

<!-- Toplevel -->

  <xsl:template match="/">
    <xsl:apply-templates select="xyacc"/>
  </xsl:template>

  <xsl:template match="xyacc">
    <xsl:apply-templates select="declarations"/>
    <xsl:value-of select="$newline"/>
    <xsl:text>%%</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:apply-templates select="rules"/>
    <xsl:value-of select="$newline"/>
  </xsl:template>


<!-- Declarations -->

  <xsl:template match="declarations">
    <xsl:apply-templates select="token_declaration|associativity_declaration|type_declaration|start_declaration|symtbl_declaration"/>
  </xsl:template>


<!-- TODO: Print out types if any -->
  <xsl:template match="token_declaration">
    <xsl:text>%token</xsl:text>
    <xsl:apply-templates select="type"/>
    <xsl:for-each select="token_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/>
  </xsl:template>

   <xsl:template match="associativity_declaration">
    <xsl:text>%</xsl:text>
    <xsl:value-of select="@direction"/>
    <xsl:for-each select="token_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/>
  </xsl:template>

   <xsl:template match="type_declaration">
    <xsl:text>%type</xsl:text>
    <xsl:apply-templates select="type"/>
    <xsl:for-each select="nonterm_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/>
  </xsl:template>
 
   <xsl:template match="start_declaration">
    <xsl:text>%start </xsl:text>
    <xsl:value-of select="nonterm_id"/>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="symtbl_declaration">
    <xsl:text>%symtbl</xsl:text>
    <xsl:for-each select="symtbl_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/> 
  </xsl:template>

<!-- Types -->

  <xsl:template match="type">
    <xsl:text> &lt;</xsl:text>
    <xsl:apply-templates select="stringty|elementty|attributety|contentty|listty"/>
    <xsl:text>&gt;</xsl:text>
  </xsl:template>

  <xsl:template match="stringty">
    <xsl:text>string</xsl:text>
  </xsl:template>

  <xsl:template match="elementty">
    <xsl:text>element</xsl:text>
  </xsl:template>

  <xsl:template match="attributety">
    <xsl:text>attribute</xsl:text>
  </xsl:template>

  <xsl:template match="contentty">
    <xsl:text>content</xsl:text>
  </xsl:template>

  <xsl:template match="listty">
    <xsl:text>list </xsl:text>
    <xsl:apply-templates select="*[1]"/>
  </xsl:template>

<!-- Rules -->

  <xsl:template match="rules">
    <xsl:apply-templates select="rule"/>
  </xsl:template>

  <xsl:template match="rule">
    <xsl:value-of select="nonterm_id"/>
    <xsl:text> : </xsl:text>
    <xsl:for-each select="match">
      <xsl:apply-templates select="."/>
      <xsl:if test="position() != last()">
        <xsl:value-of select="$newline"/>
        <xsl:text> | </xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:value-of select="$newline"/>
    <xsl:text>;</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="error_match">
    <xsl:text>error</xsl:text>
  </xsl:template>

  <xsl:template match="symbol">
    <xsl:apply-templates select="token_id|nonterm_id|char"/>
    <xsl:text> </xsl:text>
  </xsl:template>

  <xsl:template match="token_id">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="nonterm_id">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="char">
    <xsl:text>'</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>'</xsl:text>
  </xsl:template>

  <xsl:template match="precedence">
    <xsl:text>%prec </xsl:text>
    <xsl:value-of select="token_id"/>
  </xsl:template>

  <xsl:template match="seq">
    <xsl:text>{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
  </xsl:template>

  <xsl:template match="action">
    <xsl:apply-templates/>
  </xsl:template>

<!-- Actions -->

  <xsl:template match="term">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="empty">
    <xsl:text>""</xsl:text>
  </xsl:template>

  <xsl:template match="concat">
    <xsl:text>concat(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="*[2]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="string">
    <xsl:text>"</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>"</xsl:text>
  </xsl:template>


  <xsl:template match="text">
    <xsl:text>text(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="text">
    <xsl:text>literal(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="element">
    <xsl:text>element(</xsl:text>
    <xsl:apply-templates select="name"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="attlist"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="content"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="attribute">
    <xsl:text>att[</xsl:text>
    <xsl:apply-templates select="name"/>
    <xsl:text>](</xsl:text>
    <xsl:value-of select="value"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="nil">
    <xsl:text>nil</xsl:text>
  </xsl:template>

  <xsl:template match="cons">
    <xsl:text>cons(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="*[2]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="append">
    <xsl:text>append(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="*[2]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>
    
  <xsl:template match="list">
    <xsl:text>list(</xsl:text>
    <xsl:for-each select="./*">
      <xsl:apply-templates select="."/>
      <xsl:if test="position() != last()">
        <xsl:text>, </xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="rev">
    <xsl:text>rev(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>
 
  <xsl:template match="error">
    <xsl:text>error(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="accept">
    <xsl:text>accept(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="var">
    <xsl:text>$</xsl:text>
    <xsl:value-of select="@num"/>
  </xsl:template>

<!-- Effects -->

  <xsl:template match="add">
    <xsl:text>ADD(</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="term"/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="remove">
    <xsl:text>REMOVE(</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="term"/>
    <xsl:text>);</xsl:text>
  </xsl:template>


</xsl:stylesheet>

