<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="text"/>

<!-- General definitions -->

  <xsl:strip-space elements="*"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>

<!-- Toplevel -->

  <xsl:template match="/">
    <xsl:apply-templates select="yacc"/>
  </xsl:template>

  <xsl:template match="yacc">
    <xsl:apply-templates select="declarations"/>
    <xsl:value-of select="$newline"/>
    <xsl:text>%%</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:apply-templates select="rules"/>
    <xsl:value-of select="$newline"/>
    <xsl:text>%%</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:apply-templates select="definitions"/>
    <xsl:value-of select="$newline"/>
  </xsl:template>


<!-- Declarations -->

  <xsl:template match="declarations">
    <xsl:apply-templates select="token_declaration|union_declaration|associativity_declaration|type_declaration|start_declaration|code_declaration"/>
  </xsl:template>


  <xsl:template match="token_declaration">
    <xsl:text>%token </xsl:text>
    <xsl:apply-templates select="type_id"/>
    <xsl:for-each select="token_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="union_declaration">
    <xsl:text>%union {</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:for-each select="union_field">
      <xsl:value-of select="type"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="type_id"/>
      <xsl:text>;</xsl:text>
      <xsl:value-of select="$newline"/>
    </xsl:for-each>
    <xsl:text>};</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>

   <xsl:template match="associativity_declaration">
    <xsl:text>%</xsl:text>
    <xsl:value-of select="@direction"/>
    <xsl:for-each select="token_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/>
  </xsl:template>

   <xsl:template match="type_declaration">
    <xsl:text>%type </xsl:text>
    <xsl:apply-templates select="type_id"/>
    <xsl:for-each select="nonterm_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="type_id">
    <xsl:text>&lt;</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>&gt; </xsl:text>
  </xsl:template>
 
  <xsl:template match="start_declaration">
    <xsl:text>%start </xsl:text>
    <xsl:value-of select="nonterm_id"/>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="code_declaration">
    <xsl:text>%{</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="."/>
    <xsl:value-of select="$newline"/>
    <xsl:text>%}</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>

<!-- Rules -->

  <xsl:template match="rules">
    <xsl:apply-templates select="rule"/>
  </xsl:template>

  <xsl:template match="rule">
    <xsl:value-of select="nonterm_id"/>
    <xsl:text> : </xsl:text>   
    <xsl:for-each select="match">
      <xsl:apply-templates select="."/>
      <xsl:if test="position() != last()">
        <xsl:value-of select="$newline"/>
	<xsl:text> | </xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:value-of select="$newline"/>
    <xsl:text>;</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="match">
    <xsl:for-each select="error_match|symbol|action">
      <xsl:apply-templates select="."/>
      <xsl:if test="position() != last()">
	<xsl:text> </xsl:text>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="error_match">
    <xsl:text>error</xsl:text>
  </xsl:template>

  <xsl:template match="symbol">
    <xsl:apply-templates select="token_id|nonterm_id|char"/>
  </xsl:template>

  <xsl:template match="token_id">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="nonterm_id">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="char">
    <xsl:text>'</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>'</xsl:text>
  </xsl:template>

  <xsl:template match="precedence">
    <xsl:text>%prec </xsl:text>
    <xsl:value-of select="token_id"/>
  </xsl:template>

  <xsl:template match="action">
    <xsl:text>{</xsl:text>
    <xsl:apply-templates select="ccode"/>
    <xsl:text>}</xsl:text>
  </xsl:template>

<!-- Definitions -->

  <xsl:template match="ccode">
    <xsl:value-of select="."/>
  </xsl:template>

    

</xsl:stylesheet>

